diff -ur nagios-3.0.4/base/utils.c nagios-3.0.4.with_notifications_timeout/base/utils.c
--- nagios-3.0.4/base/utils.c	2008-10-15 18:43:55.000000000 +0100
+++ nagios-3.0.4.with_notifications_timeout/base/utils.c	2008-10-31 07:40:30.000000000 +0000
@@ -575,12 +575,30 @@
 		if(result<-1 || result>3)
 			result=STATE_UNKNOWN;
 
-		/* initialize output */
-		strcpy(buffer,"");
-
 		/* initialize dynamic buffer */
 		dbuf_init(&output_dbuf,dbuf_chunk);
 
+		/* Opsera patch to check timeout before attempting to read output via pipe. Originally by Sven Nierlein */
+		/* Removed bytes_read from if statement below as not applicable */
+		/* if there was a critical return code AND the command time exceeded the timeout thresholds, assume a timeout */
+		if(result==STATE_CRITICAL && (end_time.tv_sec-start_time.tv_sec)>=timeout){
+
+			/* set the early timeout flag */
+			*early_timeout=TRUE;
+
+			/* try to kill the command that timed out by sending termination signal to child process group */
+			kill((pid_t)(-pid),SIGTERM);
+			sleep(1);
+			kill((pid_t)(-pid),SIGKILL);
+		        }
+
+		/* Create if ! timeout block here, so read output if timeout has not occurred */
+		/* Indentation not fixed to allow easier patching */
+		else {
+
+		/* initialize output */
+		strcpy(buffer,"");
+
 		/* try and read the results from the command output (retry if we encountered a signal) */
 		do{
 			bytes_read=read(fd[0],buffer,sizeof(buffer)-1);
@@ -613,28 +631,18 @@
 		if(output!=NULL && output_dbuf.buf)
 			*output=(char *)strdup(output_dbuf.buf);
 
-		/* free memory */
-		dbuf_free(&output_dbuf);
-
-		/* if there was a critical return code and no output AND the command time exceeded the timeout thresholds, assume a timeout */
-		if(result==STATE_CRITICAL && bytes_read==-1 && (end_time.tv_sec-start_time.tv_sec)>=timeout){
-
-			/* set the early timeout flag */
-			*early_timeout=TRUE;
-
-			/* try to kill the command that timed out by sending termination signal to child process group */
-			kill((pid_t)(-pid),SIGTERM);
-			sleep(1);
-			kill((pid_t)(-pid),SIGKILL);
-		        }
+		} /* End if ! timeout */
 
-		log_debug_info(DEBUGL_COMMANDS,1,"Execution time=%.3f sec\n, early timeout=%d, result=%d",*exectime,*early_timeout,result);
+		log_debug_info(DEBUGL_COMMANDS,1,"Execution time=%.3f sec, early timeout=%d, result=%d, output=%s\n",*exectime,*early_timeout,result,(output_dbuf.buf==NULL)?"(null)":output_dbuf.buf);
 
 #ifdef USE_EVENT_BROKER
 		/* send data to event broker */
-		broker_system_command(NEBTYPE_SYSTEM_COMMAND_END,NEBFLAG_NONE,NEBATTR_NONE,start_time,end_time,*exectime,timeout,*early_timeout,result,cmd,(output==NULL)?NULL:*output,NULL);
+		broker_system_command(NEBTYPE_SYSTEM_COMMAND_END,NEBFLAG_NONE,NEBATTR_NONE,start_time,end_time,*exectime,timeout,*early_timeout,result,cmd,(output_dbuf.buf==NULL)?NULL:output_dbuf.buf,NULL);
 #endif
 
+		/* free memory */
+		dbuf_free(&output_dbuf);
+
 		/* close the pipe for reading */
 		close(fd[0]);
 	        }
